function PwgDataLoader( map, opts )
{
	this._map = map;
	this.options = Object.extend(
		{
			reload_data_timeout: 1800,
			rectangle_of_confusion: G_DEFAULT_ICON.iconSize
		}
		, opts || {} );
	this.options.rectangle_of_confusion = new GSize( this.options.rectangle_of_confusion.width, this.options.rectangle_of_confusion.height );
	if (this.options.rectangle_of_confusion.width<16) this.options.rectangle_of_confusion.width=16;
	if (this.options.rectangle_of_confusion.height<16) this.options.rectangle_of_confusion.height=16;
}


// returns the number of digits (between 0 and 6) to round a lat/lon so that it does not vary by more than factor*pow(-exp)
getLatLonDigits = function(d, factor, exp)
{
	if (d<0) return getLatLonDigits(-d,factor,exp);
	var digits = Math.ceil( exp - Math.log(d*factor)/Math.LN10 );
	return digits<0 ? 0 : (digits>6 ? 6 : digits);
}

Math.roundN = function(d, digits) { var c = Math.pow(10,digits); return Math.round(d*c)/c; }


PwgDataLoader.prototype =  {


_urlMapData: null,
_timerReloadData: null,
_dataLoading: false,

_previousLoadDataReq : {box: null, zoom:0, resultBounds:null},

start: function(urlMapData)
{
	this._urlMapData = urlMapData;
	GEvent.bind( this._map, "movestart", this, this.clearTimerReloadData );
	GEvent.bind( this._map, "moveend", this, this._onMapMoveEnd );
	this._loadData();
},

terminate: function()
{
	this.clearTimerReloadData();
	this._map = null;
},

clearTimerReloadData : function()
{
	if (this._timerReloadData)
	{
		clearTimeout( this._timerReloadData );
		this._timerReloadData = null;
		return true;
	}
	return false;
},

_onMapMoveEnd: function()
{
	this.clearTimerReloadData();
	this._timerReloadData = setTimeout(  this._onTimeoutLoadData.bind(this), this.options.reload_data_timeout );
},


_onTimeoutLoadData: function ()
{
	if (this._dataLoading) return; // still in progress
	this.clearTimerReloadData();
	this._loadData();
},

_loadData: function()
{
	var bounds = this._map.getBounds();

	// BEGIN BUG in maps api 2.185 up to "today" 2.193 - when map wraps horizontally more than 360 deg - the getBounds is wrong
	/*if ( this._map.getZoom() <= 2 || this._map.getZoom() == 3 && this._map.getSize().width > 1000 )
	{
		bounds = new GLatLngBounds(
				new GLatLng( bounds.getSouthWest().lat(),  -180 ),
				new GLatLng( bounds.getNorthEast().lat(),  180 )
			);
	}*/
	// END BUG
	var latRange = bounds.toSpan().lat();
	var latPrec = latRange * this.options.rectangle_of_confusion.height / this._map.getSize().height;

	var lonRange = bounds.toSpan().lng();
	var lonPrec = ( lonRange>=0 ? lonRange : 360-lonRange )* this.options.rectangle_of_confusion.width / this._map.getSize().width;

	if ( this._previousLoadDataReq.box!=null )
	{ // not the first time
		if ( this._previousLoadDataReq.box.containsBounds( bounds ) )
		{
			if ( this._previousLoadDataReq.resultBounds == null )
				return; // no images here
			if ( this._map.getZoom() <= this._previousLoadDataReq.zoom )
				return;
			else
			{
				if (
					this._map.getZoom() == this._previousLoadDataReq.zoom + 1
					&& this._map.getZoom() < this._map.getBoundsZoomLevel(this._previousLoadDataReq.resultBounds)
					)
				{
					if (document.is_debug) GLog.write('no load: zoom crt: '+this._map.getZoom()+'; prev: '+this._previousLoadDataReq.zoom+'; target: '+this._map.getBoundsZoomLevel(this._previousLoadDataReq.resultBounds));
					return;
				}
			}
		}
		else
		{
		}
	}

	var nd=0, sd=0, ed=0, wd=0;
	if ( !bounds.isFullLat() )
	{
		nd = latRange*12/100;
		sd = latRange*4/100;
	}
	if ( !bounds.isFullLng() )
	{
		ed = lonRange*9/100;
		wd = lonRange*7/100;
	}
	var digits = Math.max( getLatLonDigits(latRange,4,2), getLatLonDigits(lonRange,4,2) );
	var box = new GLatLngBounds(
			new GLatLng( Math.roundN(bounds.getSouthWest().lat()-sd,digits), Math.roundN( bounds.getSouthWest().lng()-wd,digits ) ),
			new GLatLng( Math.roundN(bounds.getNorthEast().lat()+nd,digits), Math.roundN( bounds.getNorthEast().lng()+ed,digits) )
		);

	var url = this._urlMapData;
	url += "&box=" + box.getSouthWest().toUrlValue(5) + "," + box.getNorthEast().toUrlValue(5);
	url += "&lap=" +  Math.roundN( latPrec, getLatLonDigits(latPrec,1,1) ).toString();
	url += "&lop=" +  Math.roundN( lonPrec, getLatLonDigits(lonPrec,1,1) ).toString();

	if (document.is_debug) {
		GLog.write( "bounds: " + this._map.getBounds().getSouthWest().toUrlValue() + " " + this._map.getBounds().getNorthEast().toUrlValue() +"; zoom: "+this._map.getZoom() +"; size: "+this._map.getSize().toString() +"; c: "+this._map.getCenter().toUrlValue() );
		GLog.writeUrl( url );
	}

	this._previousLoadDataReq.box = box;
	this._previousLoadDataReq.zoom = this._map.getZoom();
	this._previousLoadDataReq.resultBounds = null;
	this._dataLoading = true;

	try {
		GEvent.trigger( this, "dataloading" );
		GDownloadUrl(url, this._onDataReceived.bind(this) );
	}
	catch (e) {
		this._dataLoading = false;
		this._previousLoadDataReq.box=null;
		GEvent.trigger( this, "dataloadfailed", 600, e );
	}
},

_onDataReceived: function(data, responseCode)
{
	var resp;
	try
	{
		if (responseCode!=200)
		{
			this._previousLoadDataReq.box=null;
			GEvent.trigger( this, "dataloadfailed", responseCode, null );
		}
		else
		try
		{
			eval('resp = ' + data);
			if (resp.nb_items == undefined)
				throw new Error( "DATA DECODING ERROR" );

			this._previousLoadDataReq.resultBounds = resp.bounds;
			if (document.is_debug && resp.debug) GLog.write( resp.debug );
			GEvent.trigger( this, "dataloaded", resp );
		}
		catch (e)	{
			this._previousLoadDataReq.box=null;
			GEvent.trigger( this, "dataloadfailed", responseCode, e );
			var s = e.message;
			s += '\n' + data.substr(0,1000);
			alert( s );
		}
	}
	finally {
		this._dataLoading = false;
	}
}

}
